@@ -160,40 +160,6 @@ html_engine_selection_pop (HTMLEngine *e)
 	html_engine_edit_selection_updater_update_now (e->selection_updater);
 }
 
-void
-html_engine_cut_and_paste_begin (HTMLEngine *e, const gchar *undo_op_name, const gchar *redo_op_name)
-{
-	html_engine_hide_cursor (e);
-	html_engine_selection_push (e);
-	html_engine_clipboard_push (e);
-	html_undo_level_begin (e->undo, undo_op_name, redo_op_name);
-	html_engine_cut (e);
-}
-
-void
-html_engine_cut_and_paste_end (HTMLEngine *e)
-{
-	if (e->clipboard) {
-		html_engine_paste (e);
-		html_engine_clipboard_clear (e);
-	}
-	html_undo_level_end (e->undo);
-	html_engine_clipboard_pop (e);
-	html_engine_selection_pop (e);
-	html_engine_show_cursor (e);
-}
-
-void
-html_engine_cut_and_paste (HTMLEngine *e, const gchar *undo_op_name, const gchar *redo_op_name,
-			   HTMLObjectForallFunc iterator, gpointer data)
-{
-	html_engine_edit_selection_updater_update_now (e->selection_updater);
-	html_engine_cut_and_paste_begin (e, undo_op_name, redo_op_name);
-	if (e->clipboard)
-		html_object_forall (e->clipboard, e, iterator, data);
-	html_engine_cut_and_paste_end (e);
-}
-
 static void
 spell_check_object (HTMLObject *o, HTMLEngine *e, gpointer data)
 {
@@ -240,10 +206,10 @@ html_is_in_word (gunichar uc)
 void
 html_engine_select_word_editable (HTMLEngine *e)
 {
-	while (html_is_in_word (html_cursor_get_prev_char (e->cursor)))
+	while (html_selection_word (html_cursor_get_prev_char (e->cursor)))
 		html_cursor_backward (e->cursor, e);
 	html_engine_set_mark (e);
-	while (html_is_in_word (html_cursor_get_current_char (e->cursor)))
+	while (html_selection_word (html_cursor_get_current_char (e->cursor)))
 		html_cursor_forward (e->cursor, e);
 }
 
@@ -644,7 +610,6 @@ html_engine_next_cell (HTMLEngine *e, gboolean create)
 			html_engine_insert_table_row (e, TRUE);
 		}
 		html_engine_show_cursor (e);
-		gtk_html_edit_make_cursor_visible (e->widget);
 
 		return TRUE;
 	}
@@ -667,7 +632,6 @@ html_engine_prev_cell (HTMLEngine *e)
 		} while (current_cell == cell);
 
 		html_engine_show_cursor (e);
-		gtk_html_edit_make_cursor_visible (e->widget);
 
 		return TRUE;
 	}
